"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var pipe_and_compose_1 = require("pipe-and-compose");
var path = require("object-path");
/**
 * deduce is a function that executes a set of deductions against an input source,
 * each deduction will be responsible for selecting one or more values from the input source
 * and specifying the destination key in the output object.
 *
 * Deductions are executed sequentially in array order, each deduction will fully
 * complete before the next deduction is executed, during the execution of the deduction
 * the reducer pipeline will be invoked with the initial source value, the final
 * result of the reducer pipeline will be stored in the destination key of hte output object
 *
 * @param input Array or Object that is the input source.
 * @param deductions An array of Deductions that will be executed sequentially.
 */
exports.deduce = function (input, deductions) {
    return deductions.reduce(function (o, _a) {
        var source = _a.source, destination = _a.destination, _b = _a.reducers, reducers = _b === void 0 ? [] : _b;
        // Create a new transform stack that will process the current value
        var transform = pipe_and_compose_1.pipeline.apply(void 0, __spreadArrays([Array.isArray(source)
                ? source.map(function (k) { return path.get(input, k); })
                : path.get(input, source),
            function (v) { return v; }], reducers));
        // Execute the pipeline and set the result to the target key on the output object
        path.set(o, destination, transform);
        return o;
    }, Object.create({}));
};
//# sourceMappingURL=index.js.map